import csv
import os
import shutil
import socketserver
import subprocess
import threading
import time
from datetime import datetime
from http.server import BaseHTTPRequestHandler
from io import StringIO, BytesIO

import cuitools
import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
from dateutil.relativedelta import relativedelta
from prompt_toolkit.shortcuts import yes_no_dialog
from texttable import Texttable

from cszp import cszp_log
from cszp import cszp_module
from cszp.cszp_module import figlet, Input

matplotlib.use('Agg')


class soccerHTTPServer_Handler(BaseHTTPRequestHandler):
    def log_message(self, format, *args):
        return

    def do_GET(self):
        if self.path == "/":
            self.send_response(200)
            self.send_header("Content-Type", "text/html")
            self.end_headers()
            try:
                with open("./html/index.html") as f:
                    self.wfile.write(f.read().encode("utf-8"))
            except FileNotFoundError as e:
                print("\t\033[38;5;3m[WARNING]", e)
                self.wfile.write("404 Not Found!".encode("utf-8"))

        elif self.path == "/html.csv":
            if result != "":
                self.send_response(200)
                self.send_header("Content-Type", "text/csv")
                self.end_headers()
                bs = StringIO(result)
                data = pd.DataFrame(list(csv.reader(bs))[1:],
                                    columns=('date', 'team1', 'team2', 'team1_score', 'team2_score', 'toku1', "toku2"))
                data = data.set_index('date')
                # print(data)
                csvd = result.splitlines()[1:]
                csvtd = []
                for i in csvd:
                    csvtd.append(i.split(","))
                csvd = ""
                for i in csvtd:
                    for j in range(len(i)):
                        if j != 0:
                            if j != len(i) - 1:
                                csvd += i[j] + ","
                            else:
                                csvd += i[j] + "\n"
                self.wfile.write(
                    ("プレイヤー1,プレイヤー２," + data["team1"][0] + "の得点," + data["team2"][0] + "の得点," + data["team1"][
                        0] + "の得失点差," +
                     data["team2"][0] + "の得失点差\n" + csvd).encode("utf-8"))
            else:
                self.send_response(404)
                self.end_headers()
        elif self.path == "/file1.png":
            self.send_response(200)
            self.send_header("Content-Type", "image/png")
            self.end_headers()
            if result != "":
                bs = StringIO(result)
                data = pd.DataFrame(list(csv.reader(bs))[1:],
                                    columns=('date', 'team1', 'team2', 'team1_score', 'team2_score', 'toku1', "toku2"))
                data["team1_score"] = data["team1_score"].astype(int)
                data["team2_score"] = data["team2_score"].astype(int)
                # print(result)
                heikin = data.mean()
                x = np.array(range(max(max(data["team1_score"]), max(data["team2_score"])) + 1))
                y = np.array([0] * (max(max(data["team1_score"]), max(data["team2_score"])) + 1))
                for i in data["team1_score"]:
                    y[i] += 1
                plt.bar(x - 0.15, y, color="y", width=0.3,
                        label=data["team1"].values[0] + "\nAverage:" + str(np.round(heikin["team1_score"], decimals=2)))

                x = np.array(range(max(max(data["team1_score"]), max(data["team2_score"])) + 1))
                y = np.array([0] * (max(max(data["team1_score"]), max(data["team2_score"])) + 1))
                for i in data["team2_score"]:
                    y[i] += 1
                plt.bar(x + 0.15, y, color="r", width=0.3,
                        label=data["team2"].values[0] + "\nAverage:" + str(np.round(heikin["team2_score"], decimals=2)))
                plt.ylabel("number of times")
                plt.xlabel("point")
                plt.legend(loc='best')
                plt.grid(True)
                img = BytesIO()
                plt.savefig(img, dpi=300, format="png")
                plt.clf()
                self.wfile.write(img.getvalue())
            else:
                self.wfile.write(open("./html/nofile.png", "rb").read())
        elif self.path == "/load.gif":
            self.send_response(200)
            self.send_header("Content-Type", "image/gif")
            self.end_headers()
            self.wfile.write(open("./html/load.gif", "rb").read())
        else:
            self.send_response(404)
            self.send_header("Content-Type", "text/html")
            self.end_headers()
            self.wfile.write("404 Not Found!".encode("utf-8"))


def totime(sec):
    return "{0.days}d {0.hours}h {0.minutes}m {0.seconds}s".format(relativedelta(seconds=sec))


class soccer:
    def server(self):
        PORT = 20000
        Handler = soccerHTTPServer_Handler
        try:
            socketserver.TCPServer.allow_reuse_address = True
            self.httpd = socketserver.ThreadingTCPServer(("", PORT), Handler)
            print("\t\033[38;5;10m\033[1m[OK]\033[0mThe web server has been started. Port:", PORT)
            self.httpd.serve_forever()
        except OSError as e:
            print("\t\033[38;5;3m[WARNING]\033[0mCould not start web server port", PORT, ":" + str(e))
            time.sleep(5)
            self.server()

    def __init__(self, cmd, lang, loop, module, logs2, Input, s_start, reset=True, exit=True):
        global result
        result = ""
        self.result_list = []
        if reset:
            cuitools.reset()

        sc = "rcssserver " + cmd[2]
        wc = "soccerwindow2 > /dev/null 2>&1"
        if logs2[3]:
            sc += " server::synch_mode=true"
        print("\t\033[1m\033[38;5;172m" + datetime.now().strftime("%Y/%m/%d %H:%M:%S"))
        print("\033[38;5;2mTeam1-CMD : " + cmd[0])
        print("Team2-CMD : " + cmd[1])
        print("\033[38;5;2mSERVER-CMD : " + sc)
        print("WINDOW-CMD : " + wc + "\033[0m")
        error = 0
        list_time = []
        if loop > 1 and s_start:
            server = threading.Thread(target=self.server)
            server.setDaemon(True)
            server.start()

        try:
            for i in range(loop):
                print("\t\033[38;5;4m[INFO]\033[0mLOOP:", i + 1, "/", loop)
                tmp_time = time.time()
                logs = "\t" + datetime.now().strftime("%Y/%m/%d %H:%M:%S") + "\n" + "SERVER-CMD : " + sc + "\n"
                logt1 = "\t" + datetime.now().strftime("%Y/%m/%d %H:%M:%S") + "\n"
                logt2 = "\t" + datetime.now().strftime("%Y/%m/%d %H:%M:%S") + "\n"
                cszp_module.killsoccer()

                data = module.Open("./config/config.conf")
                datas = data.read().splitlines()[0]
                data.close()
                datas = datas.split(",")

                if datas[1] == "on":
                    try:
                        subprocess.Popen(wc, shell=True)
                    except subprocess.CalledProcessError as e:
                        print("\033[38;5;3m[WARNING]\033[0m" + "SOCCERWINDOW2_ERROR" + str(e))

                try:
                    logtemp1 = subprocess.Popen(sc, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                    print("\t\033[38;5;4m[INFO]\033[0mrcssserver start.")
                    logtemp2 = subprocess.Popen(cmd[0], shell=True, stdout=subprocess.PIPE,
                                                stderr=subprocess.PIPE)
                    print("\t\033[38;5;4m[INFO]\033[0mteam1 start.")
                    time.sleep(1)
                    logtemp3 = subprocess.Popen(cmd[1], shell=True, stdout=subprocess.PIPE,
                                                stderr=subprocess.PIPE)
                    print("\t\033[38;5;4m[INFO]\033[0mteam2 start.")
                    stdout, stderr = logtemp2.communicate()
                    try:
                        stderr = stderr.decode("utf-8")
                        stdout = stdout.decode("utf-8")
                    except UnicodeDecodeError as e:
                        print("\033[38;5;3m[WARNING]\033[0m\t" + str(e))
                        stderr = stderr.decode("utf-8", "replace")
                        stdout = stdout.decode("utf-8", "replace")
                    logt1 += stdout + stderr
                    logt1 += "\n"
                    print("\t\033[38;5;4m[INFO]\033[0mreturncode:" + str(logtemp2.returncode))
                    if logtemp2.returncode != 0:
                        logtemp3.kill()
                        logtemp1.kill()
                        print("\t\033[38;5;9m\033[1m[ERR]\033[0m" + lang.lang("コマンド実行中にエラーが発生しました。"))
                        error = 1
                        continue

                    print("\t\033[38;5;4m[INFO]\033[0mteam1 stop.")
                    stdout, stderr = logtemp3.communicate()
                    try:
                        stderr = stderr.decode("utf-8")
                        stdout = stdout.decode("utf-8")
                    except UnicodeDecodeError as e:
                        print("\033[38;5;3m[WARNING]\033[0m\t" + str(e))
                        stderr = stderr.decode("utf-8", "replace")
                        stdout = stdout.decode("utf-8", "replace")
                    logt2 += stdout + stderr
                    logt2 += "\n"
                    print("\t\033[38;5;4m[INFO]\033[0mreturncode:" + str(logtemp3.returncode))
                    if logtemp3.returncode != 0:
                        logtemp1.kill()
                        print("\t\033[38;5;9m\033[1m[ERR]\033[0m" + lang.lang("コマンド実行中にエラーが発生しました。"))
                        error = 1
                        continue

                    print("\t\033[38;5;4m[INFO]\033[0mteam2 stop.")
                    stdout, stderr = logtemp1.communicate()
                    try:
                        stderr = stderr.decode("utf-8")
                        stdout = stdout.decode("utf-8")
                    except UnicodeDecodeError as e:
                        print("\033[38;5;3m[WARNING]\033[0m\t" + str(e))
                        stderr = stderr.decode("utf-8", "replace")
                        stdout = stdout.decode("utf-8", "replace")
                    logs += stdout + stderr
                    logs += "\n"
                    print("\t\033[38;5;4m[INFO]\033[0mreturncode:" + str(logtemp1.returncode))
                    if logtemp1.returncode != 0:
                        logtemp1.kill()
                        print("\t\033[38;5;9m\033[1m[ERR]\033[0m" + lang.lang("コマンド実行中にエラーが発生しました。"))
                        error = 1
                        continue

                    print("\t\033[38;5;4m[INFO]\033[0mrcssserver stop.")
                except (subprocess.CalledProcessError, FileNotFoundError) as e:
                    print("\t\033[38;5;9m\033[1m[ERR]\033[0m" + lang.lang("コマンド実行中にエラーが発生しました。\n"), e)
                    cszp_module.killsoccer(False)
                    error = 1
                    continue
                except (EOFError, KeyboardInterrupt):
                    cszp_module.killsoccer(False)
                    error = 1
                    if not exit:
                        raise KeyboardInterrupt
                    break

                logs += "\t" + datetime.now().strftime("%Y/%m/%d %H:%M:%S")
                logt1 += "\t" + datetime.now().strftime("%Y/%m/%d %H:%M:%S")
                logt2 += "\t" + datetime.now().strftime("%Y/%m/%d %H:%M:%S")
                exittime = datetime.now().strftime("%Y%m%d_%H%M%S")
                cszp_module.killsoccer(False)
                soccer = cszp_log.log(logs)
                result += "\n" + exittime + "," + soccer[0] + "," + soccer[1] + "," + soccer[2] + "," + soccer[
                    3] + "," + \
                          str(int(soccer[2]) - int(soccer[3])) + "," + str(int(soccer[3]) - int(soccer[2]))
                self.result_list.append(
                    [exittime, soccer[0], soccer[1], int(soccer[2]), int(soccer[3]), int(soccer[2]) - int(soccer[3]),
                     int(soccer[3]) - int(soccer[2])])

                if logs2[0]:
                    data = module.Open("./config/config.conf")
                    df = data.read()
                    # print(df)
                    temp = open(df.split(",")[7] + "/" + exittime + "_server.log", "w")
                    temp.write(logs)
                    temp.close()

                if logs2[1]:
                    data = module.Open("./config/config.conf")
                    df = data.read()
                    # print(df.split("'"))
                    temp = open(df.split(",")[7] + "/" + exittime + "_team1.log", "w")
                    temp.write(logt1)
                    temp.close()
                    temp = open(df.split(",")[7] + "/" + exittime + "_team2.log", "w")
                    temp.write(logt2)
                    temp.close()

                if logs2[2] is not False:
                    data = module.Open("./config/config.conf")
                    df = data.read()
                    # print(df.split(",")[9] + "/" + file)
                    temp = open(df.split(",")[7] + "/" + logs2[2], "w")
                    soccer = cszp_log.log(logs)
                    temp.write(
                        "\n" + exittime + "," + soccer[0] + "," + soccer[1] + "," + soccer[2] + "," + soccer[
                            3] + "," + str(
                            int(soccer[2]) - int(soccer[3])) + "," + str(int(soccer[3]) - int(soccer[2])))
                    temp.close()

                list_time.append(time.time() - tmp_time)
                print("━" * 50 + "\n\033[38;5;4m[INFO]\033[0mRun Time:", totime(int(list_time[-1])), " Total",
                      totime(int(sum(list_time))), " ETA",
                      totime(int((loop - len(list_time)) * (sum(list_time) / len(list_time)))), "\n" + "━" * 50)

            if error == 0:
                exittime = datetime.now().strftime("%Y%m%d_%H%M%S")
                subprocess.check_call("mkdir ./html_logs/" + exittime, shell=True)
                subprocess.check_call("cp ./html/index.html ./html_logs/" + exittime + "/", shell=True)
                print("\033[38;5;10m\033[1m[OK]\033[0mDone! time:" + totime(
                    int(sum(list_time))) + "\n" + "━" * 50 + "\nresult")
                bs = StringIO(result)
                data = pd.DataFrame(list(csv.reader(bs))[1:],
                                    columns=(
                                        'date', 'team1', 'team2', 'team1_score', 'team2_score', 'team1 goal difference',
                                        "team2 goal difference"))
                data = data.set_index('date')
                # print(data)
                csvd = result.splitlines()[1:]
                csvtd = []
                for i in csvd:
                    csvtd.append(i.split(","))
                csvd = ""
                for i in csvtd:
                    for j in range(len(i)):
                        if j != 0:
                            if j != len(i) - 1:
                                csvd += i[j] + ","
                            else:
                                csvd += i[j] + "\n"
                with open("./html_logs/" + exittime + "/" + "html.csv", "w") as f:
                    f.write("プレイヤー1,プレイヤー２," + data["team1"][0] + "の得点," + data["team2"][0] + "の得点," + data["team1"][
                        0] + "の得失点差," +
                            data["team2"][0] + "の得失点差\n" + csvd)
                data["team1_score"] = data["team1_score"].astype(int)
                data["team2_score"] = data["team2_score"].astype(int)
                print(data)
                print("━" * 50)
                heikin = data.mean()
                x = np.array(range(max(max(data["team1_score"]), max(data["team2_score"])) + 1))
                y = np.array([0] * (max(max(data["team1_score"]), max(data["team2_score"])) + 1))
                for i in data["team1_score"]:
                    y[i] += 1
                plt.bar(x - 0.15, y, color="y", width=0.3,
                        label=data["team1"].values[0] + "\nAverage:" + str(np.round(heikin["team1_score"], decimals=2)))

                x = np.array(range(max(max(data["team1_score"]), max(data["team2_score"])) + 1))
                y = np.array([0] * (max(max(data["team1_score"]), max(data["team2_score"])) + 1))
                for i in data["team2_score"]:
                    y[i] += 1
                plt.bar(x + 0.15, y, color="r", width=0.3,
                        label=data["team2"].values[0] + "\nAverage:" + str(np.round(heikin["team2_score"], decimals=2)))
                plt.ylabel("number of times")
                plt.xlabel("point")
                plt.legend(loc='best')
                plt.grid(True)
                plt.savefig("./html_logs/" + exittime + "/" + "file1.png", dpi=300)
                plt.clf()

        finally:
            try:
                self.httpd.shutdown()
            except AttributeError:
                pass
            if exit:
                Input.Input(lang.lang("Enterキーを押して続行..."), dot=False)

    def get_result(self):
        return self.result_list


def check_path(text, module):
    if not os.path.isfile(text[1][1]):
        if not text[1][1] in module.hogo:
            text[1][2] = 0
        else:
            text[1][2] = 1
    else:
        text[1][2] = 1
    if not os.path.isfile(text[2][1]):
        if not text[2][1] in module.hogo:
            text[2][2] = 0
        else:
            text[2][2] = 1
    else:
        text[2][2] = 1
    if text[1][1] == text[2][1]:
        text[1][2] = 0
        text[2][2] = 0
    return text


def yesno(title, text):
    tmp = 0
    k = ""
    while k != "\n":
        if tmp == 0:
            cuitools.box(title,
                         text + ["\033[7mYes\033[0m  No"])
        else:
            cuitools.box(title,
                         text + ["Yes  \033[7mNo\033[0m"])
        k = cuitools.Key()
        if k == "\x1b":
            k = cuitools.Key()
            if k == "[":
                k = cuitools.Key()
                if k == "C":
                    if tmp == 0:
                        tmp = 1
                elif k == "D":
                    if tmp == 1:
                        tmp = 0
    return tmp


def kakunin(module):
    data = module.Open("./config/config.conf")
    datas = data.read().splitlines()
    data.close()
    datas = datas[0].split(",")
    datal = []
    for i in range(0, len(datas), 2):
        datat = []
        datat += datas[i:i + 2]
        datal.append(datat)
    table = Texttable()
    table.add_rows(datal)
    return table


def setting(lang, module, Input_, testmode=False, loopmode=False):
    data = module.Open("./config/config.conf")
    datas = data.read().splitlines()
    data.close()
    datas = datas[0].split(",")
    table = kakunin(module)

    if yes_no_dialog(
            title=lang.lang("cszp 簡単サッカー実行プログラム") + "/" + lang.lang("設定確認"),
            text=table.draw() + lang.lang("\nEnterキーを押して続行...")).run():
        k = ""
        ok = False
        select = 0
        text = [
            ["\033[38;5;10m" + lang.lang("前ページへ戻る(ホーム)"), False],
            ["\033[38;5;11m" + lang.lang("Team1(黄色)チームのパスを選択"), "None", 0],
            ["\033[38;5;9m" + lang.lang("Team2(赤色)チームのパスを選択"), "None", 0],
            ["\033[38;5;10m" + "─" * shutil.get_terminal_size()[0], None],
            ["\033[38;5;12m" + lang.lang("サーバーの引数を入力（ない場合は空欄）"), "", None],
            ["\033[38;5;10m" + "─" * shutil.get_terminal_size()[0], None],
            ["\033[38;5;9m" + lang.lang("保存先") + " " + datas[7], None],
            ["\033[38;5;11m" + lang.lang("サーバーログの保存"), "No", None],
            ["\033[38;5;11m" + lang.lang("プレイヤーログの保存"), "No", None],
            ["\033[38;5;11m" + lang.lang("csvログの保存"), "No", None],
            ["\033[38;5;10m" + "─" * shutil.get_terminal_size()[0], None],
            ["\033[38;5;14m" + lang.lang("synchモードでの実行"), "No", None],
            ["\033[38;5;14m" + lang.lang("サーバーの起動"), "Yes", None],
        ]
        if loopmode:
            text += [
                ["\033[38;5;10m" + "─" * shutil.get_terminal_size()[0], None],
                ["\033[38;5;15m" + lang.lang("ループ回数を指定"), 3, 1]
            ]
        if testmode:
            text += [
                ["", None],
                ["\033[38;5;11m" + lang.lang("*テストモードで実行しています"), None]
            ]
        else:
            text += [["", None]]
        text += [
            ["\033[38;5;10m" + lang.lang("サッカーを実行"), False]
        ]
        ver = open("./version")
        v = ver.read()
        ver.close()
        while k != "\n" or ok is False:
            cuitools.reset()
            print("\033[1;1H\033[0m\033[38;5;172m")
            figlet("cszp " + v)
            for i, j in enumerate(text):
                if j[1] is False:
                    if i == select:
                        print("\033[1m\033[38;5;9m> " + j[0] + "\033[0m")
                    else:
                        print("  " + j[0] + "\033[0m")
                elif j[1] is not None:
                    if j[2] is None:
                        if i == select:
                            print(
                                "\033[1m\033[38;5;9m> " + j[0] + ":\033[0m\033[4m\033[38;5;14m" + str(j[1]) + "\033[0m")
                        else:
                            print("  " + j[0] + ":\033[0m\033[4m\033[38;5;14m" + str(j[1]) + "\033[0m")
                    else:
                        if i == select:
                            print(
                                "\033[1m\033[38;5;9m> " + j[0] + ":\033[0m\033[4m\033[38;5;14m" + str(
                                    j[1]) + "\033[0m " + (
                                        "\033[38;5;9m✖" * ((j[2] - 1) * -1) + "\033[38;5;10m✔" * j[2]) + "\033[0m ")
                        else:
                            print("  " + j[0] + ":\033[0m\033[4m\033[38;5;14m" + str(j[1]) + "\033[0m " + (
                                    "\033[38;5;9m✖" * ((j[2] - 1) * -1) + "\033[38;5;10m✔" * j[2]) + "\033[0m ")
                else:
                    print(j[0])
            print("\n")

            k = cuitools.Key()
            if k == "\n":
                if select == 0:
                    break
                elif select == 1 or select == 2:
                    tmp = yesno(lang.lang("選択"), [lang.lang("簡単サッカー実行リストから選択しますか？")])
                    if tmp == 0:
                        data = module.Open("./config/setting.conf")
                        datas = data.read().splitlines()[0]
                        data.close()
                        datas = datas.split(",")
                        datal = []
                        for i in range(2, len(datas), 2):
                            datal.append((datas[i + 1], datas[i]))
                        if len(datal) == 0:
                            k = ""
                            while k != "\n":
                                cuitools.box(lang.lang("エラー"), [
                                    lang.lang("簡単サッカー実行リストにチームが設定されていないのでこの機能は使用できません。"),
                                    lang.lang("settingコマンドでチームの追加ができます。詳しくはsettingコマンドを確認してください。"),
                                    lang.lang("Enterキーを押して続行...")
                                ])
                                k = cuitools.Key()
                        else:
                            tmp = Input()
                            tmp = tmp.Input(lang.lang("簡単サッカー実行リストで設定した名前を指定（Tabで一覧が確認できます）"),
                                            word=list(map(lambda n: n[1], datal)))
                            text[select][1] = "None"
                            text = check_path(text, module)
                            for i in datal:
                                if i[1] == tmp:
                                    text[select][1] = i[0]
                                    text = check_path(text, module)
                                    break
                    else:
                        if select == 1:
                            tmp = cuitools.Inputfilegui(lang.lang("Team1(黄色)チームのパスを選択") + lang.lang("(選択方法の変更・終了はQキー)"))
                        else:
                            tmp = cuitools.Inputfilegui(lang.lang("Team2(赤色)チームのパスを選択") + lang.lang("(選択方法の変更・終了はQキー)"))
                        if tmp == -1:
                            pass
                        elif tmp == -2:
                            raise KeyboardInterrupt
                        else:
                            text[select][1] = tmp
                            text = check_path(text, module)
                elif select == 4:
                    tmp = Input()
                    text[select][1] = tmp.Input(lang.lang("サーバーの引数を入力（ない場合は空欄）"))

                elif select == 7:
                    tmp = yesno(lang.lang("選択"), [lang.lang("サーバーログを保存しますか？")])
                    if tmp == 0:
                        text[select][1] = "Yes"
                    else:
                        text[select][1] = "No"
                elif select == 8:
                    tmp = yesno(lang.lang("選択"), [lang.lang("プレイヤーログを保存しますか？")])
                    if tmp == 0:
                        text[select][1] = "Yes"
                    else:
                        text[select][1] = "No"
                elif select == 9:
                    tmp = yesno(lang.lang("選択"), [lang.lang("csvログを保存しますか？")])
                    if tmp == 0:
                        tmp = Input()
                        tmp = tmp.Input("filename")
                        if len(tmp.split(".")) == 1:
                            tmp += ".csv"
                        text[select][1] = tmp
                    else:
                        text[select][1] = "No"
                elif select == 11:
                    tmp = yesno(lang.lang("選択"), [lang.lang("synchモードで実行しますか？")])
                    if tmp == 0:
                        text[select][1] = "Yes"
                    else:
                        text[select][1] = "No"
                elif select == 12:
                    tmp = yesno(lang.lang("選択"), [lang.lang("サーバーを起動させますか？")])
                    if tmp == 0:
                        text[select][1] = "Yes"
                    else:
                        text[select][1] = "No"
                elif select == 14 and loopmode:
                    tmp = Input()
                    tmp = tmp.Input(lang.lang("ループ回数を指定"))
                    try:
                        text[select][1] = int(tmp)
                    except ValueError:
                        text[select][1] = 0
                    if text[select][1] == 0:
                        text[select][2] = 0
                    else:
                        text[select][2] = 1
                elif select == len(text) - 1:
                    ok = True
                    for i in text:
                        if len(i) == 3:
                            if i[2] == 0:
                                k = ""
                                ok = False
                                while k != "\n":
                                    cuitools.box(lang.lang("エラー"), [lang.lang("問題箇所があります。確認してください。"),
                                                                    lang.lang("Enterキーを押して続行...")])
                                    k = cuitools.Key()
                                break

            if k == "\x1b":
                k = cuitools.Key()
                if k == "[":
                    k = cuitools.Key()
                    if k == "B":
                        if len(text) - 1 > select:
                            select += 1
                            while text[select][1] is None:
                                select += 1
                    elif k == "A":
                        if 0 < select:
                            select -= 1
                            while text[select][1] is None:
                                select -= 1

        if ok:
            data = module.Open("./config/config.conf")
            datas = data.read().splitlines()
            data.close()
            datas = datas[0].split(",")
            arg = text[4][1]
            arg += "server::auto_mode=true server::kick_off_wait=20 server::game_over_wait=20 " \
                   "server::connect_wait=2000 server::game_log_dir=" + datas[7] + " server::text_log_dir=" + datas[
                       7] + " server::game_logging=" + (
                           "true" * (datas[3] == "on") + "false" * (datas[3] == "off")) + " server::text_logging=" + (
                           "true" * (datas[3] == "on") + "false" * (datas[3] == "off"))

            if testmode:
                arg += " server::nr_normal_halfs=1 server::nr_extra_halfs=0 server::penalty_shoot_outs=0 " \
                       "server::half_time=10"
            if loopmode:
                loop = text[14][1]
            else:
                loop = 1
            if text[12][1] == "Yes":
                s = True
            else:
                s = False
            if text[11][1] == "Yes":
                synch = True
            else:
                synch = False
            if text[9][1] == "Yes":
                tmp1 = True
            else:
                tmp1 = False
            if text[8][1] == "Yes":
                tmp2 = True
            else:
                tmp2 = False
            if text[7][1] == "Yes":
                tmp3 = True
            else:
                tmp3 = False
            soccer([text[2][1], text[1][1], arg], lang, loop, module, [tmp3, tmp2, tmp1, synch],
                   Input_, s)


def rrt(lang, module, Input_):
    table = kakunin(module)

    data = module.Open("./config/setting.conf")
    datas = data.read().splitlines()[0]
    data.close()
    datas = datas.split(",")
    datal = []
    for i in range(2, len(datas), 2):
        datal.append((datas[i + 1], datas[i]))
    if len(datal) == 0:
        k = ""
        while k != "\n":
            cuitools.box(lang.lang("エラー"), [
                lang.lang("簡単サッカー実行リストにチームが設定されていないのでこの機能は使用できません。"),
                lang.lang("settingコマンドでチームの追加ができます。詳しくはsettingコマンドを確認してください。"),
                lang.lang("Enterキーを押して続行...")
            ])
            k = cuitools.Key()
    elif yes_no_dialog(
            title=lang.lang("cszp 簡単サッカー実行プログラム") + "/" + lang.lang("設定確認"),
            text=table.draw() + lang.lang("\nEnterキーを押して続行...")).run():
        k = ""
        csv_save = False
        ok = False
        ver = open("./version")
        v = ver.read()
        ver.close()
        kantan_list = list(map(lambda n: n[1], datal))
        kantan_cmd = list(map(lambda n: n[0], datal))
        select = 0
        selector = []
        synch = False
        while k != "\n" or not ok:
            cuitools.reset()
            print("\033[1;1H\033[0m\033[38;5;172m")
            figlet("cszp " + v)
            if select == -1:
                print("\033[1m\033[38;5;9m>  \033[38;5;10m" + lang.lang("前ページへ戻る(ホーム)") + "\033[0m\n")
            else:
                print("   \033[38;5;10m" + lang.lang("前ページへ戻る(ホーム)") + "\033[0m\n")
            print("\033[38;5;12m" + lang.lang("総当たり戦をさせるチームを選択してください。"))
            for i, j in enumerate(kantan_list):
                if i in selector:
                    if select == i:
                        print("\033[1m\033[38;5;9m>\033[38;5;11m* \033[38;5;10m" + j + "\033[0m")
                    else:
                        print("\033[38;5;11m * \033[38;5;10m" + j + "\033[0m")
                elif select == i:
                    print("\033[1m\033[38;5;9m>  \033[38;5;10m" + j + "\033[0m")
                else:
                    print("   \033[38;5;10m" + j + "\033[0m")
            print("")
            if select == len(kantan_list):
                print("\033[1m\033[38;5;9m>  \033[38;5;14m" + lang.lang(
                    "synchモードでの実行") + ":\033[0m\033[4m\033[38;5;14m" + "No" * (
                              (synch - 1) * -1) + "Yes" * synch + "\033[0m")
            else:
                print("   \033[38;5;14m" + lang.lang("synchモードでの実行") + ":\033[0m\033[4m\033[38;5;14m" + "No" * (
                        (synch - 1) * -1) + "Yes" * synch + "\033[0m")
            if select == len(kantan_list) + 1:
                if csv_save is False:
                    print("\033[1m\033[38;5;9m>  \033[38;5;11m" + lang.lang("csvログの保存") + ":\033[0m\033[4m\033["
                                                                                          "38;5;14mNo\033[0m")
                else:
                    print("\033[1m\033[38;5;9m>  \033[38;5;11m" + lang.lang(
                        "csvログの保存") + ":\033[0m\033[4m\033[38;5;14m" + csv_save + "\033[0m")
            else:
                if csv_save is False:
                    print(
                        "   \033[38;5;11m" + lang.lang("csvログの保存") + ":\033[0m\033[4m\033[38;5;14mNo\033[0m")
                else:
                    print(
                        "   \033[38;5;11m" + lang.lang(
                            "csvログの保存") + ":\033[0m\033[4m\033[38;5;14m" + csv_save + "\033[0m")
            if select == len(kantan_list) + 2 and len(selector) > 1:
                print("\033[1m\033[38;5;9m>  \033[38;5;10m" + lang.lang("サッカーを実行") + "\033[0m")
            elif len(selector) < 2:
                print("   \033[38;5;243m" + lang.lang("サッカーを実行") + "\033[0m")
            else:
                print("   \033[38;5;10m" + lang.lang("サッカーを実行") + "\033[0m")

            table = []
            tmp = [""]
            for i in selector:
                tmp.append(kantan_list[i])
            table.append(tmp)
            for i in selector:
                table.append([kantan_list[i]] + [""] * len(selector))
            print(Texttable().add_rows(table).draw())
            k = cuitools.Key()
            if k == "\x1b":
                k = cuitools.Key()
                if k == "[":
                    k = cuitools.Key()
                    if k == "B":
                        if len(kantan_list) + 2 > select:
                            select += 1
                            if len(kantan_list) + 2 == select and len(selector) < 2:
                                select -= 1
                    elif k == "A":
                        if -1 < select:
                            select -= 1
            if (k == " " or k == "\n") and select == -1:
                break
            elif (k == " " or k == "\n") and select == len(kantan_list) + 2:
                ok = True
                break
            elif (k == " " or k == "\n") and select == len(kantan_list) + 1:
                tmp = yesno(lang.lang("選択"), [lang.lang("csvログを保存しますか？")])
                if tmp == 0:
                    tmp = Input()
                    tmp = tmp.Input("filename")
                    if len(tmp.split(".")) == 1:
                        tmp += ".csv"
                    csv_save = tmp
                else:
                    csv_save = False
            elif (k == " " or k == "\n") and select == len(kantan_list):
                tmp = yesno(lang.lang("選択"), [lang.lang("synchモードで実行しますか？")])
                if tmp == 0:
                    synch = True
                else:
                    synch = False
            elif (k == " " or k == "\n") and select > -1:
                if select in selector:
                    for i, j in enumerate(selector):
                        if j == select:
                            del selector[i]
                            break
                else:
                    selector.append(select)
        if ok:
            table = []
            tmp = [""]
            for i in selector:
                tmp.append(kantan_list[i])
            table.append(tmp)
            for i in selector:
                table.append([kantan_list[i]] + [""] * len(selector))
            for i in range(len(table)):
                if i != 0:
                    table[i][i] = "N/A"
            tmp = []
            for i in selector:
                for j in selector:
                    if kantan_cmd[i] != kantan_cmd[j] and not [kantan_cmd[j], kantan_cmd[i]] in tmp:
                        tmp.append([kantan_cmd[i], kantan_cmd[j]])
            tmp2 = []
            for i in selector:
                for j in selector:
                    if kantan_cmd[i] != kantan_cmd[j] and not [kantan_list[j], kantan_list[i]] in tmp2:
                        tmp2.append([kantan_list[i], kantan_list[j]])
            # print(tmp)
            data = module.Open("./config/config.conf")
            datas = data.read().splitlines()
            data.close()
            datas = datas[0].split(",")
            arg = "server::auto_mode=true server::kick_off_wait=20 server::game_over_wait=20 " \
                  "server::connect_wait=2000 server::game_log_dir=" + datas[7] + " server::text_log_dir=" + datas[
                      7] + " server::game_logging=" + (
                          "true" * (datas[3] == "on") + "false" * (datas[3] == "off")) + " server::text_logging=" + (
                          "true" * (datas[3] == "on") + "false" * (datas[3] == "off"))
            # arg += " server::nr_normal_halfs=1 server::nr_extra_halfs=0 server::penalty_shoot_outs=0 " \
            #        "server::half_time=10"
            s = [0, 1]
            results = []
            for i, j in zip(tmp, tmp2):
                cuitools.reset()
                print("\033[1;1H\033[0m\033[38;5;172m")
                figlet("cszp " + v)
                print("\033[0m")
                print(Texttable().add_rows(table).draw() + "\n" + "━" * 50)
                try:
                    _ = soccer([i[0], i[1], arg], lang, 1, module, [False, False, False, synch],
                               Input_, False, False, False)
                except KeyboardInterrupt:
                    break
                result = _.get_result()
                if len(result) == 0:
                    table[s[0] + 1][s[1] + 1] = "Error"
                else:
                    tmp3 = result[0]
                    tmp3[1] = table[s[0] + 1][0]
                    tmp3[2] = table[0][s[1] + 1]
                    results.append(tmp3)
                    table[s[0] + 1][s[1] + 1] = str(result[0][3]) + "-" + str(result[0][4])
                s[1] += 1
                if s[1] > len(selector) - 1:
                    s[0] += 1
                    s[1] = 0
                tmp3 = list(map(lambda n: n + 1, s))
                while [j[1], j[0]] != [table[tmp3[0]][0], table[0][tmp3[1]]]:
                    print([table[tmp3[0]][0], table[0][tmp3[1]]], [j[1], j[0]])
                    tmp3[1] += 1
                    if tmp3[1] > len(selector):
                        tmp3[0] += 1
                        tmp3[1] = 0
                if len(result) == 0:
                    table[tmp3[0]][tmp3[1]] = "Error"
                else:
                    table[tmp3[0]][tmp3[1]] = str(result[0][4]) + "-" + str(result[0][3])

                while s[1] <= s[0]:
                    s[1] += 1
                    if s[1] > len(selector):
                        s[0] += 1
                        s[1] = 0
            if csv_save is not False:
                with open(datas[7] + "/" + csv_save, "w") as f:
                    f.write("\n".join(list(map(lambda n: ",".join(map(str, n)), results))))
            cuitools.reset()
            print("\033[1;1H\033[0m\033[38;5;172m")
            figlet("cszp " + v)
            print("\033[0m")
            print("\033[1m\033[38;5;11m" + lang.lang("集計結果") + "\033[0m\n" + "━" * 50 + "\n" + Texttable().add_rows(
                table).draw())
            tmp = []
            for i in results:
                if not i[1] in list(map(lambda n: n[0], tmp)):
                    tmp.append([i[1], [0, 0, 0]])
                for j, k in enumerate(tmp):
                    if k[0] == i[1]:
                        break
                if i[3] > i[4]:
                    tmp[j][1][0] += 1
                elif i[3] < i[4]:
                    tmp[j][1][1] += 1
                else:
                    tmp[j][1][2] += 1

                if not i[2] in list(map(lambda n: n[0], tmp)):
                    tmp.append([i[2], [0, 0, 0]])
                for j, k in enumerate(tmp):
                    if k[0] == i[2]:
                        break
                if i[4] > i[3]:
                    tmp[j][1][0] += 1
                elif i[4] < i[3]:
                    tmp[j][1][1] += 1
                else:
                    tmp[j][1][2] += 1
            table = [["", "W", "D", "L", "Total"]]
            for i in tmp:
                table.append([i[0], i[1][0], i[1][2], i[1][1], i[1][0] - i[1][1]])
            print("\n" + "━" * 50 + "\n" + Texttable().add_rows(table).draw())
            Input_.Input(lang.lang("Enterキーを押して続行..."), dot=False)



import cuitools
from cszp import to256_window



def Help(lang):
    help_file_name = ""
    while help_file_name != "***":
        help_list = {
            "cszpへようこそ！": "welcome_to_cszp",
            "基本的な使い方": "basic_usage",
            "start・test・loopコマンドの使い方": "basic_usage_2",
            "前ページへ戻る(ホーム)":"***"
        }
        keys = list(map(lang.lang,list(help_list.keys())))
        index = cuitools.printlist(lang.lang("cszpのヘルプの参照"),keys)
        help_file_name = list(help_list.values())[index]
        if help_file_name != "***":
            with open("./docs/" + help_file_name + "_" + lang.enable_lang_name + ".md") as f:
                to256_window(keys[index] + " / " + lang.lang("qキーで終了"), f.read(),
                             background_file="./docs/background.png")



import glob
import json
import os
import shutil
import subprocess
import zipfile

import cuitools
from prompt_toolkit.shortcuts import button_dialog
from tqdm import tqdm

import cszp.cszp_module
from cszp.cszp_module import terminal


def plugin_text(lang, text):
    return list(map(lambda n: json.load(open(n + "/setup.json"))[text], lang.pluginlist))


def gen_rand_str_hex(length):
    import os
    import hashlib
    buf = ''
    while len(buf) < length:
        buf += hashlib.md5(os.urandom(100)).hexdigest()
    return buf[0:length]


def plugin(module, lang):
    print("\033[0m")
    select = cuitools.printlist(lang.lang("プラグインを選択"), plugin_text(lang, "name") + [
        lang.lang("バイナリファイルデータの追加・変更"), lang.lang("プラグインを追加"), lang.lang("前ページへ戻る(ホーム)")])
    if select != -1 and select < len(plugin_text(lang, "name")):
        sselect = 0
        while True:

            printtext = [
                lang.lang("名前:") + plugin_text(lang, "name")[select],
                lang.lang("バージョン:") + plugin_text(lang, "version")[select],
                lang.lang("管理者:") + plugin_text(lang, "author")[select],
                lang.lang("管理者のメールアドレス:") + plugin_text(lang, "author_email")[select],
                ""
            ]
            if sselect == 0:
                printtext.append(">> " + lang.lang("前ページへ戻る(ホーム)"))
                printtext.append("   " + lang.lang("説明を読む"))
            else:
                printtext.append("   " + lang.lang("前ページへ戻る(ホーム)"))
                printtext.append(">> " + lang.lang("説明を読む"))
            # print(printtext)
            cuitools.box(lang.lang("詳細情報") + " " + plugin_text(lang, "name")[select], printtext)
            k = cuitools.Key()
            if k == "\x1b":
                cuitools.Key()
                k = cuitools.Key()
                if k == "B":
                    if sselect < 1:
                        sselect += 1
                elif k == "A":
                    if sselect > 0:
                        sselect -= 1
            elif k == "\n":
                if sselect == 0:
                    break
                else:
                    cuitools.printlist(lang.lang("説明を読む"), plugin_text(lang, "description")[select].splitlines(), False)
    elif select == len(plugin_text(lang, "name")):
        while True:
            cuitools.reset()
            print(lang.lang("必要なデータをダウンロード中..."))
            data_list = cszp_module.download_file_mem(lang,
                                                      "https://raw.githubusercontent.com/kumitatepazuru/Soccer-binary"
                                                      "-files-for-cszp "
                                                      "/master/data.json")
            if data_list is not None:
                data_list = json.loads(data_list)
                text_list = list(map(lambda n: n["text"], data_list))
                folder_list = list(map(lambda n: n["folder"], data_list))
                url_list = list(map(lambda n: n["url"], data_list))
                team_data_list = list(map(lambda n: n["data"], data_list))
                num = cuitools.printlist(lang.lang("バイナリファイルデータの追加・変更"), text_list + [lang.lang("前ページへ戻る")])
                if num == len(data_list):
                    break
                else:
                    while True:
                        if os.path.isfile("config/" + folder_list[num]):
                            result = button_dialog(
                                title=lang.lang("詳細情報") + "/" + text_list[num],
                                text="download URL:" + url_list[num] + "\n" + lang.lang("インストール済み"),
                                buttons=[
                                    (lang.lang('前ページへ戻る'), True),
                                    (lang.lang("選手一覧"), None)
                                ]
                            ).run()
                            if result:
                                break
                            elif result is None:
                                cuitools.reset()
                                print("\n".join(json.loads(cszp_module.download_file_mem(lang, team_data_list[num]))))
                                input(lang.lang("Enterキーを押して続行..."))
                        else:
                            result = button_dialog(
                                title=lang.lang("詳細情報") + "/" + text_list[num],
                                text="download URL:" + url_list[num] + "\n" + lang.lang("未インストール"),
                                buttons=[
                                    (lang.lang('前ページへ戻る'), True),
                                    (lang.lang("インストールする"), False),
                                    (lang.lang("選手一覧"), None)
                                ]
                            ).run()
                            if result:
                                break
                            elif result is False:
                                cuitools.reset()
                                print(lang.lang("必要なデータをダウンロード中...") + lang.lang("これには数分~数十分かかることがあります。") + "\n")
                                print(url_list[num])
                                cszp_module.download_files_progress(lang, [url_list[num]])
                                print("\n" + lang.lang("データを解凍中...") + lang.lang("これには数分~数十分かかることがあります。"))
                                h = ""
                                while h == "" or os.path.isdir("teams/" + h):
                                    h = gen_rand_str_hex(8)
                                with zipfile.ZipFile("/tmp/" + folder_list[num] + ".zip") as existing_zip:
                                    existing_zip.extractall("/tmp")
                                try:
                                    cmd = ["unzip","-uqd","/tmp","/tmp/" + folder_list[num] + ".zip"]
                                    print(" ".join(cmd))
                                    subprocess.check_call(cmd)
                                except FileNotFoundError:
                                    while True:
                                        cuitools.box(lang.lang("エラー"),[lang.lang("この機能を使用するにはunzipをインストールする必要があります。"),lang.lang("Enterキーを押して続行...")])
                                        if cuitools.Key() == "\n":
                                            break
                                    continue
                                with open(
                                        "/tmp/Soccer-binary-files-for-cszp-" + folder_list[num] + "/hogo.json") as f:
                                    with open("config/hogo.json") as F:
                                        data = json.load(F)
                                        tmp = json.load(f)
                                        tmp = list(map(lambda n: "LD_LIBRARY_PATH=$LD_LIBRARY_PATH:./teams/" + h +
                                                                 "/libs ./teams/" + h + "/" + n, tmp))
                                        data += tmp
                                    with open("config/hogo.json", "w") as F:
                                        json.dump(data, F)
                                    with open(
                                            "/tmp/Soccer-binary-files-for-cszp-" + folder_list[
                                                num] + "/team.json") as F:
                                        tmp2 = json.load(F)
                                        data = ""
                                        for i, j in zip(tmp2, tmp):
                                            data += "," + i + "," + j
                                        F = module.Open("./config/plus.txt", "a")
                                        F.write(data)
                                os.mkdir("teams/" + h)
                                print("\n" + lang.lang("データをコピー中..."))
                                for i in tqdm(glob.glob(
                                        "/tmp/Soccer-binary-files-for-cszp-" + folder_list[num] + "/teams/*")):
                                    shutil.copytree(i, "teams/" + h + "/" + i.split("/")[-1])
                                os.mkdir("teams/" + h + "/libs")
                                for i in tqdm(
                                        glob.glob("/tmp/Soccer-binary-files-for-cszp-" + folder_list[num] + "/libs/*")):
                                    shutil.copy(i, "teams/" + h + "/libs")
                                with open("config/" + folder_list[num], "w") as f:
                                    f.write(h)
                                # module = cszp_module.Open()
                                input("\n" + lang.lang("Enterキーを押して続行..."))

                            elif result is None:
                                cuitools.reset()
                                print("\n".join(json.loads(cszp_module.download_file_mem(lang, team_data_list[num]))))
                                input("\n" + lang.lang("Enterキーを押して続行..."))
                cuitools.reset()
                print("")
            else:
                break

        plugin(module, lang)
    elif select == len(plugin_text(lang, "name")) + 1:
        k = ""
        while k != "\n":
            cuitools.box(lang.lang("プラグインを追加"), [
                lang.lang("プラグインの追加を行います。"),
                lang.lang("プラグインの入ったzipファイルを選択してください。"),
                lang.lang("終了したい場合は次の画面に行ってからqキーを押してください。"),
                lang.lang("Enterキーを押して続行...")
            ])
            k = cuitools.Key()
        ok = None
        while ok is None:
            path = cuitools.Inputfilegui(lang.lang("プラグインの入ったzipファイルを選択（終了したい場合はQキー）"))
            if path == -1:
                plugin(module, lang)
                break
            elif path == -2:
                raise KeyboardInterrupt()
            else:
                cuitools.reset()
                try:
                    with zipfile.ZipFile(path) as existing_zip:
                        existing_zip.extractall('./plugins/')
                except zipfile.BadZipFile:
                    k = ""
                    while k != "\n":
                        cuitools.box(lang.lang("エラー"), [
                            lang.lang("zipファイルではありません。"),
                            lang.lang("Enterキーを押して続行...")
                        ], reset_=True)
                        k = cuitools.Key()
                else:
                    k = ""
                    lang = terminal(noenter=True)
                    lang.autostart(lang)
                    while k != "\n":
                        cuitools.box("完了", [
                            lang.lang("正常にプラグインが追加されました。"),
                            lang.lang("正常にプラグインが動作するか確認してください。"),
                            lang.lang("Enterキーを押して続行...")
                        ])
                        k = cuitools.Key()
                    ok = lang
        return ok

    elif select == len(plugin_text(lang, "name")) + 2:
        pass



print("\033[38;5;2mloading Now...")

import os
print("1/12")
import subprocess
print("\033[1A2/12")
import sys
print("\033[1A3/12")

sys.path.append(os.getcwd())
os.chdir(os.path.abspath(os.path.dirname(__file__)))

import json
print("\033[1A4/12")
import locale
print("\033[1A5/12")
import platform
print("\033[1A6/12")
import shutil
print("\033[1A7/12")

from cszp.cszp_module import Open, terminal, figlet
print("\033[1A8/12")
from cszp.cszp_markdown import to256_window
print("\033[1A9/12")
from cszp import cszp_menu
print("\033[1A10/12")
import cuitools as subp
print("\033[1A11/12")
import cszp.cszp_module
print("\033[1A12/12")

subp.reset()
print("\033[1;1H\033[0m\033[38;5;172m")
ver = open("./version")
figlet("cszp " + ver.read())


module = Open()


def main():
    # noinspection PyBroadException
    print("")
    if int(sys.version_info[0]) == 3 and int(sys.version_info[1]) >= 6 and int(sys.version_info[2]) >= 1:
        sys.stdout.write("\033[38;5;10m\033[1m[OK] ")
        print("\033[0m\033[38;5;2mpythonVersion\033[38;5;7m:\033[38;5;6m" + platform.python_version())
    else:
        print("\033[0m\033[38;5;2mpythonVersion\033[38;5;7m:\033[38;5;6m" + platform.python_version())
        print("\033[38;5;9m\033[1m[ERR] \033[0mThis program requires python3.6.1 or higher version.")
        sys.exit()

    if not os.path.isdir("config"):
        print("\033[38;5;4m[INFO]\033[0mCreate directory: config")
        os.mkdir("config")
    data = module.Open("./config/config.conf")
    path = data.read().splitlines()[0].split(",")[7]
    data.close()
    if not os.path.isdir("html_logs"):
        print("\033[38;5;4m[INFO]\033[0mCreate directory: html_logs")
        os.mkdir("html_logs")
    if not os.path.isdir(path):
        print("\033[38;5;4m[INFO]\033[0mCreate directory: csvdata")
        os.mkdir(path)
    if not os.path.isdir("plugins"):
        print("\033[38;5;4m[INFO]\033[0mCreate directory: plugins")
        os.mkdir("plugins")
    if not os.path.isdir("teams"):
        print("\033[38;5;4m[INFO]\033[0mCreate directory: teams")
        os.mkdir("teams")

    # main

    if not os.path.isfile("lang"):
        file = open("language/lang.json")
        lang_list = json.load(file)
        file.close()
        langf = open("lang", "w")
        langn = [k for k, n in lang_list.items() if n == locale.getlocale()[0].lower() + ".lang"]
        if len(langn) == 1:
            langf.write(langn[0])
        else:
            langf.write("1")
        langf.close()
    lang = terminal()
    lang.autostart(lang)
    Input = cszp.cszp_module.Input()
    if not os.path.isfile("start"):
        with open("start", "w") as f:
            f.write("1")
        with open("./docs/welcome_to_cszp_" + lang.enable_lang_name + ".md") as f:
            to256_window(lang.lang("cszpへようこそ！") + " / " + lang.lang("qキーで終了"), f.read())
    try:
        cszp_menu.menu(lang, module, Input)
    except KeyboardInterrupt:
        pass
    except EOFError:
        pass
    except Exception:
        import traceback
        temp = "CSZP PROGRAM ERROR\ncszp=" + open("version").read() + "\n"
        file = open("./errorlog.log", "w")
        temp += "---------- error log ----------\n" + traceback.format_exc() + "\n"
        temp += "---------- computer information ----------\nwhich python3 : " + shutil.which(
            'python3') + "\n" + "\n".join(map(lambda n: "=".join(n), list(os.environ.items())))
        temp += "\n\n---------- file list ----------\n" + subprocess.check_output("ls -al", shell=True).decode("utf-8")
        file.write(temp)
        file.close()
        print("\033[0m")
        subp.box(lang.lang("エラー"),
                 [temp.splitlines()[0], lang.lang("ログを確認してください"), "", os.getcwd() + "/errorlog.log", "",
                  lang.lang("Enterキーを押して続行...")])
        k = ""
        while k != "\n":
            k = subp.Key()

    #  stop
    subp.reset()
    print("\033[38;5;10mgood bye!")


if __name__ == '__main__':
    main()



from .extra import plugin_url, plugin_strikethrough
from .footnotes import plugin_footnotes
from .table import plugin_table
from .task_lists import plugin_task_lists


PLUGINS = {
    'url': plugin_url,
    'strikethrough': plugin_strikethrough,
    'footnotes': plugin_footnotes,
    'table': plugin_table,
    'task_lists': plugin_task_lists,
}

__all__ = [
    'PLUGINS',
    'plugin_url', 'plugin_strikethrough',
    'plugin_footnotes',
    'plugin_table',
]



def log(data):
    try:
        logdata = data.find("vs")
        while data[logdata] != "\t":
            logdata -= 1
        team1s = logdata + 2

        logdata += 2
        while data[logdata] != "'":
            logdata += 1
        team1e = logdata

        logdata += 1
        while data[logdata] != "'":
            logdata += 1
        team2s = logdata + 1

        logdata += 1
        while data[logdata] != "'":
            logdata += 1
        team2e = logdata

        team1n = data[team1s:team1e]
        team2n = data[team2s:team2e]

        while data[logdata] != " ":
            logdata += 1

        logdata += 1
        team1s = logdata

        while data[logdata] != " ":
            logdata += 1

        team1e = logdata
        logdata += 1

        while data[logdata] != " ":
            logdata += 1

        logdata += 1
        team2s = logdata

        while data[logdata] != "\n":
            logdata += 1

        team2e = logdata

        team1p = data[team1s:team1e]
        team2p = data[team2s:team2e]
        return team1n, team2n, team1p, team2p
    except IndexError:
        return "", "", "0", "0"



from mistune.scanner import escape_url
from mistune.inline_parser import ESCAPE

__all__ = ['plugin_url', 'plugin_strikethrough']


#: url link like: ``https://lepture.com/``
URL_LINK_PATTERN = r'''(https?:\/\/[^\s<]+[^<.,:;"')\]\s])'''


def parse_url_link(self, m, state):
    return 'link', escape_url(m.group(0))


def plugin_url(md):
    md.inline.register_rule('url_link', URL_LINK_PATTERN, parse_url_link)
    md.inline.rules.append('url_link')


#: strike through syntax looks like: ``~~word~~``
STRIKETHROUGH_PATTERN = (
    r'~~(?=[^\s~])('
    r'(?:\\~|[^~])*'
    r'(?:' + ESCAPE + r'|[^\s~]))~~'
)


def parse_strikethrough(self, m, state):
    text = m.group(1)
    return 'strikethrough', self.render(text, state)


def render_html_strikethrough(text):
    return '<del>' + text + '</del>'


def plugin_strikethrough(md):
    md.inline.register_rule(
        'strikethrough', STRIKETHROUGH_PATTERN, parse_strikethrough)

    index = md.inline.rules.index('codespan')
    if index != -1:
        md.inline.rules.insert(index + 1, 'strikethrough')
    else:  # pragma: no cover
        md.inline.rules.append('strikethrough')

    if md.renderer.NAME == 'html':
        md.renderer.register('strikethrough', render_html_strikethrough)



import os
import shutil
import subprocess
import sys
from importlib import import_module, reload

import cuitools as subp
from texttable import *
from cszp.cszp_module import figlet


def setting(lang, module, Input):
    subp.reset()
    inp = ""
    while not lang.searchcmd("setting", inp):
        subprocess.check_call("clear", shell=True)
        print("\033[0m\033[38;5;172m")
        v = open("./version")
        figlet("cszp " + v.read())
        v.close()
        print("\n\n\033[1m\033[38;5;10m" + lang.lang("cszp 簡単サッカー実行プログラム"))
        print("\n\033[38;5;39m" + lang.lang("簡単サッカー実行リスト"))
        # noinspection PyBroadException
        data = module.Open("./config/setting.conf")
        datas = data.read()
        data.close()
        datas = datas.split(",")
        datal = []
        for i in range(0, len(datas), 2):
            datat = []
            datat += datas[i:i + 2]
            datal.append(datat)
        table = Texttable()
        table.add_rows(datal)
        print(table.draw() + "\n\n")

        print("\033[38;5;39m" + lang.lang("設定"))
        # noinspection PyBroadException
        data = module.Open("./config/config.conf")
        datas = data.read()
        data.close()
        datas = datas.split(",")
        datal = []
        for i in range(0, len(datas), 2):
            datat = []
            datat += datas[i:i + 2]
            datal.append(datat)
        table = Texttable()
        table.add_rows(datal)
        print(table.draw() + "\n\n")

        q = lang.question("setting", "※  注 [文字列] は引数を表します。 文字列は引数名です。")
        inp = Input.Input(q[0], dot=False, normal=False, word=q[1])
        if not lang.searchcmd("setting", inp):
            print("\033[38;5;9m" + lang.lang("ERR:そのようなコマンドはありません。"))
            Input.Input(lang.lang("Enterキーを押して続行..."), dot=False)

    try:
        if inp == "back":
            pass

        elif inp.split(' ')[0] == "add":
            data = open("./config/setting.conf", "a")
            data.write("," + inp.split(' ')[1] + "," + inp.split(' ')[2])
            data.close()
            setting(lang, module, Input)

        elif inp.split(' ')[0] == "remove":
            data = module.Open("./config/setting.conf")
            datas = data.read()
            datas = datas.split(",")
            data.close()
            i = 0
            try:
                try:
                    while datas[i] != inp.split(" ")[1]:
                        i += 2
                except:
                    raise TypeError("data_ERROR")
                if datas[i+1] in module.hogo:
                    raise AttributeError("protected")
                del datas[i:i + 2]
                datat = datas
                datas = ""
                for i in datat:
                    datas += i + ","
                datas = datas[:len(datas) - 1]
                data = open("./config/setting.conf", "w")
                data.write(datas)
                data.close()
            except TypeError:
                print("\033[38;5;9m" + lang.lang("ERR:名前"), inp.split(" ")[1], lang.lang(
                    "は簡単サッカー実行リストに登録されていません。\nタイプミスを確認してください"))
                Input.Input(lang.lang("Enterキーを押して続行..."), dot=False)
            except AttributeError:
                print("\033[38;5;9m" + lang.lang("ERR:削除しようとしているリストの項目は保護されています。"))
                Input.Input(lang.lang("Enterキーを押して続行..."), dot=False)
            setting(lang, module, Input)

        elif inp.split(' ')[0] == "soccerwindow2":
            data = module.Open("./config/config.conf")
            datas = data.read()
            datas = datas.split(",")
            data.close()
            if inp.split(' ')[1].lower() == "on":
                datas[1] = "on"
            elif inp.split(' ')[1].lower() == "off":
                datas[1] = "off"
            else:
                print("\033[38;5;9m" + lang.lang("ERR:使える引数はONまたはOFFです。\nタイプミスを確認してください"))
                Input.Input(lang.lang("Enterキーを押して続行..."), dot=False)
            datat = datas
            datas = ""
            for i in datat:
                datas += i + ","
            datas = datas[:len(datas) - 1]
            data = open("./config/config.conf", "w")
            data.write(datas)
            data.close()
            setting(lang, module, Input)

        elif inp.split(' ')[0] == "rcg":
            data = module.Open("./config/config.conf")
            datas = data.read()
            datas = datas.split(",")
            data.close()
            if inp.split(' ')[1].lower() == "on":
                datas[3] = "on"
            elif inp.split(' ')[1].lower() == "off":
                datas[3] = "off"
            else:
                print("\033[38;5;9mERR:使える引数はONまたはOFFです。\nタイプミスを確認してください")
                Input.Input(lang.lang("Enterキーを押して続行..."), dot=False)
            datat = datas
            datas = ""
            for i in datat:
                datas += i + ","
            datas = datas[:len(datas) - 1]
            data = open("./config/config.conf", "w")
            data.write(datas)
            data.close()
            setting(lang, module, Input)

        elif inp.split(' ')[0] == "rcl":
            data = module.Open("./config/config.conf")
            datas = data.read()
            datas = datas.split(",")
            data.close()
            if inp.split(' ')[1].lower() == "on":
                datas[5] = "on"
            elif inp.split(' ')[1].lower() == "off":
                datas[5] = "off"
            else:
                print("\033[38;5;9mERR:使える引数はONまたはOFFです。\nタイプミスを確認してください")
                Input.Input(lang.lang("Enterキーを押して続行..."), dot=False)
            datat = datas
            datas = ""
            for i in datat:
                datas += i + ","
            datas = datas[:len(datas) - 1]
            data = open("./config/config.conf", "w")
            data.write(datas)
            data.close()
            setting(lang, module, Input)

        elif inp.split(' ')[0] == "fileout":
            data = module.Open("./config/config.conf")
            datas = data.read()
            datas = datas.split(",")
            data.close()
            datas[7] = inp.split(" ")[1]
            datat = datas
            datas = ""
            for i in datat:
                datas += i + ","
            datas = datas[:len(datas) - 1]
            data = open("./config/config.conf", "w")
            data.write(datas)
            data.close()
            setting(lang, module, Input)
        else:
            sys.path.append(lang.functo("setting", inp)[0][0])
            plugin = import_module(lang.functo("setting", inp)[1][0])
            reload(plugin)
            try:
                plugin.plugin(lang, inp)
            except Exception:
                import traceback
                temp = "PLUGIN ERROR\ncszp=" + open("version").read() + "\n"
                file = open("./errorlog.log", "w")
                temp += "---------- error log ----------\n" + traceback.format_exc() + "\n"
                temp += "---------- computer information ----------\nwhich python3 : " + shutil.which(
                    'python3') + "\n" + "\n".join(map(lambda n: "=".join(n), list(os.environ.items())))
                temp += "\n\n---------- file list ----------\n" + subprocess.check_output("ls -al", shell=True).decode(
                    "utf-8")
                file.write(temp)
                file.close()
                print("\033[0m")
                subp.box(lang.lang("エラー"), [temp.splitlines()[0], lang.lang("ログを確認してください"), "",
                                            os.getcwd() + "/errorlog.log", "", lang.lang("Enterキーを押して続行...")])
                k = ""
                while k != "\n":
                    k = subp.Key()
            setting(lang, module, Input)
    except IndexError:
        print("\033[38;5;9m" + lang.lang("ERR:引数がありません。タイプミスを確認してください"))
        Input.Input(lang.lang("Enterキーを押して続行..."), dot=False)
        setting(lang, module, Input)



import pygments.util
import cuitools
import shutil
import numpy as np
import mistune
from PIL import Image
from pygments import highlight
from pygments.lexers import get_lexer_by_name
from pygments.formatters import Terminal256Formatter


class md_code:
    def __init__(self, text, lang=None):
        self.text = text
        if lang == "":
            self.lang = None
        else:
            self.lang = lang

    def tohtml(self):
        _max = max(list(map(cuitools.subp.width_kana, self.text.split("\n")))) + 7
        if self.lang is None:
            return "━" * _max + "\n⣿ " + "\n⣿ ".join(self.text.split("\n")) + "\n" + "━" * _max
        else:
            try:
                lexer = get_lexer_by_name(self.lang, stripall=True)
            except pygments.util.ClassNotFound:
                return "━" * _max + "\n⣿ " + "\n⣿ ".join(self.text.split("\n")) + "\n" + "━" * _max
            formatter = Terminal256Formatter(encoding='utf-8')
            return "━" * _max + "\n⣿ " + "\n⣿ ".join(
                highlight(self.text, lexer, formatter).decode("utf-8").split("\n")) + "\n" + "━" * _max


class to256_renderer(mistune.HTMLRenderer):
    def text(self, text):
        return text

    def link(self, link, text=None, title=None):
        if text is None:
            return link
        else:
            return "\033[38;5;2m" + text + ":" + link + "\033[0m"

    def image(self, src, alt="", title=None):
        if alt == "":
            return "\n\033[38;5;3m" + src + "\033[0m\n"
        else:
            return "\n\033[38;5;3m" + alt + ":" + src + "\033[0m\n"

    def emphasis(self, text):
        return "\033[3m" + text + "\033[0m"

    def strong(self, text):
        return "\033[1m" + text + "\033[0m"

    def codespan(self, text):
        return "\033[48;5;16m\033[38;5;1m" + text + "\033[0m"

    def linebreak(self):
        return "\n"

    def newline(self):
        return ""

    def inline_html(self, html):
        return html

    def paragraph(self, text):
        return text + "\n"

    def heading(self, text, level):
        if level == 1:
            return "\033[1m\033[38;5;9m" + text + "  " + "\033[0m\n\033[38;5;9m" + "━" * cuitools.subp.width_kana(
                text) + "\033[0m\n"
        elif level == 2:
            return "  \033[38;5;6m" + text + "  " + "\033[0m\n\033[38;5;6m  " + "─" * cuitools.subp.width_kana(
                text) + "\033[0m\n"
        elif level == 3:
            return "    \033[4m\033[1m\033[38;5;14m" + text + "\033[0m\n"
        elif level == 4:
            return "      \033[1m\033[38;5;2m" + text + "\033[0m\n"
        elif level == 5:
            return "        \033[38;5;11m" + text + "\033[0m\n"
        else:
            return "          \033[38;5;3m" + text + "\033[0m\n"

    def thematic_break(self):
        return "\n━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━\n"

    def block_text(self, text):
        return text

    def block_code(self, code, lang=None):
        code = md_code(code, lang)
        return code.tohtml() + "\n"

    def block_quote(self, text):
        return "⣿ " + "\n⣿ ".join(text.split("\n")) + "\n"

    def block_html(self, html):
        return html + "\n"

    def block_error(self, html):
        return "\033[38;5;1m" + html + "\033[0m\n"

    def list(self, text, ordered, level, start=None):
        return text + '\n'

    def list_item(self, text, level):
        return "\033[1m\033[38;5;4m# \033[0m " + text + '\n'

    def strikethrough(self, text):
        return "\033[9m" + text + "\033[0m"


def to256(text):
    markdown = mistune.create_markdown(renderer=to256_renderer(), plugins=['strikethrough'])
    return markdown(text)


def reload(text):
    terminal_size = shutil.get_terminal_size()
    tmp = []
    for i in to256(text).split("\n"):
        j = 1
        while len(i) > j:
            length = 0
            old_j = j
            while length < terminal_size[0] - 18 and len(i) > j:
                length = cuitools.subp.width_kana(i[old_j - 1:j])
                j += 1
            if len(i) > j:
                tmp.append(i[old_j - 1:j - 1])
            else:
                tmp.append(i[old_j - 1:j])
    return tmp


def scale_to_height(img, height):
    width = round(img.width * height / img.height)
    return img.resize((width, height))


def background(file):
    img = Image.open(file)
    terminal_size = list(shutil.get_terminal_size())
    img.thumbnail(terminal_size)
    img = np.array(img)
    for no, i in enumerate(img):
        print("")
        if no == 0:
            print("\033[1;1H", end="")
        for j in i:
            print("\033[48;2;" + str(j[0]) + ";" + str(j[1]) + ";" + str(j[2]) + "m  ", end="")
        print("\033[0m", end="")


def to256_window(title, text, reset=True, background_file=None, place="c"):
    terminal_size = shutil.get_terminal_size()
    if terminal_size[0] < 15:
        cuitools.reset()
        print("\033[1;1HThe screen is too small. Please make it bigger.")
        while terminal_size[0] < 15:
            terminal_size = shutil.get_terminal_size()

    tmp = reload(text)

    if reset:
        cuitools.reset()
    if background_file is not None:
        background(background_file)
    k = ""
    scroll = 0
    while k != "q":
        terminal_size = shutil.get_terminal_size()
        printtext = tmp[0 + scroll:terminal_size[1] - 4 + scroll]
        printtext.insert(0, title)
        printtext.append("")
        lentext = terminal_size[0] - 15
        if place == "c":
            y = int(terminal_size[1] / 2 - len(printtext) / 2)
            x = int(terminal_size[0] / 2 - lentext / 2)
        elif place == "n":
            y = 1
            x = int(terminal_size[0] / 2 - lentext / 2)
        elif place == "nw":
            y = 1
            x = 1
        elif place == "ne":
            y = 1
            x = terminal_size[0] - lentext - 1
        elif place == "e":
            y = int(terminal_size[1] / 2 - len(printtext) / 2)
            x = terminal_size[0] - lentext - 1
        elif place == "w":
            y = int(terminal_size[1] / 2 - len(printtext) / 2)
            x = 1
        elif place == "s":
            y = terminal_size[1] - len(printtext)
            x = int(terminal_size[0] / 2 - lentext / 2)
        elif place == "sw":
            y = terminal_size[1] - len(printtext)
            x = 1
        elif place == "se":
            y = terminal_size[1] - len(printtext)
            x = terminal_size[0] - lentext - 1
        else:
            raise IndexError("placeはc,n,nw,ne,e,w,s,sw,seのみ対応しています")
        for i in range(len(printtext)):
            if i == 0:
                print(
                    "\033[" + str(y + i) + ";" + str(x) + "H\033[0m┏" + cuitools.subp.ljust_kana(printtext[i], lentext,
                                                                                                 "━") + "┓")
            elif i == len(printtext) - 1:
                print("\033[" + str(y + i) + ";" + str(x) + "H┗" + cuitools.subp.ljust_kana(printtext[i], lentext,
                                                                                            "━") + "┛")
            else:
                print("\033[" + str(y + i) + ";" + str(x) + "H┃" + cuitools.subp.ljust_kana(printtext[i], lentext,
                                                                                            " ") + "┃")

        k = cuitools.Key()
        old_terminal_size = terminal_size
        terminal_size = shutil.get_terminal_size()
        if old_terminal_size != terminal_size:
            if scroll > len(tmp) - terminal_size[1] + 4:
                scroll = len(tmp) - terminal_size[1] + 1
            cuitools.reset()
            tmp = reload(text)
            if background_file is not None:
                background(background_file)
        if k == "\x1b":
            k = cuitools.Key()
            if k == "[":
                k = cuitools.Key()
                if k == "A":
                    if scroll != 0:
                        scroll -= 1
                if k == "B":
                    if scroll < len(tmp) - terminal_size[1] + 4:
                        scroll += 1

if __name__ == '__main__':
    background("docs/background.png")